@@ -24,11 +24,13 @@
 import sys
 import os
 import logging
+import signal
 import utils
 import encrypt
 import eventloop
 import tcprelay
 import udprelay
+import asyncdns
 
 
 def main():
@@ -50,15 +52,26 @@ def main():
         logging.info("starting local at %s:%d" %
                      (config['local_address'], config['local_port']))
 
-        tcp_server = tcprelay.TCPRelay(config, True)
-        udp_server = udprelay.UDPRelay(config, True)
+        dns_resolver = asyncdns.DNSResolver()
+        tcp_server = tcprelay.TCPRelay(config, dns_resolver, True)
+        udp_server = udprelay.UDPRelay(config, dns_resolver, True)
         loop = eventloop.EventLoop()
+        dns_resolver.add_to_loop(loop)
         tcp_server.add_to_loop(loop)
         udp_server.add_to_loop(loop)
+
+        def handler(signum, _):
+            logging.warn('received SIGQUIT, doing graceful shutting down..')
+            tcp_server.close(next_tick=True)
+            udp_server.close(next_tick=True)
+        signal.signal(signal.SIGQUIT, handler)
         loop.run()
     except (KeyboardInterrupt, IOError, OSError) as e:
         logging.error(e)
-        os._exit(0)
+        if config['verbose']:
+            import traceback
+            traceback.print_exc()
+        os._exit(1)
 
 if __name__ == '__main__':
     main()
